
# -*- coding: UTF-8 -*-
import os


from . import DATAYES

__author__ = 'frank'


from . import api_base
try:
    from StringIO import StringIO
except:
    from io import StringIO
import pandas as pd
import sys
import logging
import traceback

import requests

username = os.environ.get('DatayesPrincipalName', 'invalid')

def MktOptionTicksHistOneDayGet(optionId, date, field = "", startSecOffset = "", endSecOffset = "", pandas = "1"):

    fieldStr = ''
    if hasattr(field,'__iter__'):
        for temp in field:
            fieldStr += temp + ","
        fieldStr = fieldStr[:-1]
    else:
        fieldStr += field

    URL = '/api/market/getOptionTicksHistOneDay.csv?field=%s&optionId=%s&date=%s&startSecOffset=%s&endSecOffset=%s' % (fieldStr, optionId, date, startSecOffset, endSecOffset)

    httpClient = api_base.__getConn__()
    csvString = api_base.__getCSV__(URL, httpClient)
    if pandas != "1":
        return csvString
    if csvString is None or len(csvString) == 0 or csvString[0] == '-':
        raise Exception(u'%s for request: %s' % (csvString, URL))
    try:

        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'optionId':'str'}, )
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def EcoDataProGet(indicID, beginDate = "", endDate = "", field = ""):
    """
    通联新宏观行业。输入指标ID，获取相关数据。

    :param indicID: 指标代码，可以通过特色数据页面获取。,可以是列表
    :param beginDate: 根据数据截止日期范围查询的开始日期，输入格式“YYYYMMDD”,可空
    :param endDate: 根据数据截止日期范围查询的结束日期，输入格式“YYYYMMDD”,可空
    :param field: 所需字段,可以是列表,可空；候选列表：indicID - 指标ID，publishDate - 数据源发布时间，periodDate - 数据期，dataValue - 数据值，updateTime - 更新时间
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
    if not api_base.is_pro_user() and not api_base.is_enterprise_user():
        raise Exception('Not uqer pro user!')
    api_name_info = DATAYES.EcoInfoProGet(indicID=indicID, field='indicID,dataApiName').drop_duplicates()
    df_list = []
    for idx, row in api_name_info.iterrows():
        thisIndicID, api_name = (row['indicID'], row['dataApiName'])
        f = getattr(DATAYES, api_name[3:] + 'Get')
        df = f(str(thisIndicID), beginDate, endDate, field)
        df_list.append(df)
    if df_list:
        return pd.concat(df_list)
    return pd.DataFrame()

if os.getenv('env', 'dev') != 'dev':
    cache_server = "tcp://db_cache_server:1111"
else:
    if os.environ.get('enterprise', '0') == '1':
        cache_server = "tcp://10.24.51.170:1111"  # stg
    else:
        cache_server = "tcp://10.22.132.36:1111"  # stg


batch_size = 5

def _get_service():
    import zerorpc
    client = zerorpc.Client(heartbeat=None, timeout=300)
    client.connect(cache_server)
    return client

def _load_factor(secID, fields, beginDate, endDate, username):
    try:
        client = _get_service()
        return 1, client.get_user_factors(secID, fields, beginDate, endDate, username)
    except Exception as e:
        error_msg = str(e)
        if "[Name Check]" in error_msg:
            raise Exception(error_msg[error_msg.find('[Name Check]')+12:])
        if '[Restriction]' in error_msg:
            raise Exception(error_msg[error_msg.find('[Restriction]')+13:])
        else:
            logging.error(traceback.format_exc().replace('\n', ''))
            raise Exception("The data is not available at this moment.")
    finally:
        client.close()

def FactorsGet(secID, beginDate, endDate, field):
    """
    获取因子数据，包括优矿因子、用户自定义因子和公共池因子
    :param secID: 股票池列表
    :param beginDate: 开始时间
    :param endDate: 结束时间
    :param field: 因子名列表，比如['PB', 'public.factor1', 'private.factor2']，不带前缀表示优矿因子，带private前缀表示用户自定义因子，带public前缀表示公共池因子
    :return: 一个pandas Panel，比如panel['PB']表示PB因子数据的DataFrame，panel['public.factor1']表示public.factor1因子数据的DataFrame，panel['private.factor2']表示private.factor2因子数据的DataFrame
    """
    from gevent.pool import Pool
    if not api_base.is_pro_user() and not api_base.is_enterprise_user():
        print ('非专业版用户，无法使用该API')
        return
    try:
        if type(field) != list:
            field = field.split(',')
    except:
        raise Exception('Invalid field format')
    client = _get_service()
    try:
        factors = client.translate_user_factors(field, username, beginDate)
    except Exception as e:
        error_msg = str(e)
        if "[Name Check]" in error_msg:
            print ("ERROR: " + error_msg[error_msg.find('[Name Check]')+12:])
            return
        if '[Restriction]' in error_msg:
            print ("ERROR: " + error_msg[error_msg.find('[Restriction]')+13:])
            return
        else:
            logging.error(traceback.format_exc().replace('\n', ''))
            raise Exception("The data is not available at this moment.")
    finally:
        client.close()
    result_factors = []
    from gevent.pool import Pool
    pool = Pool(batch_size)
    workers = []
    for key in factors:
        for f_id in factors[key]:
            factor = factors[key][f_id]
            result_factors.append(factor)

    group_factors = api_base.splist(result_factors, 3)
    for group in group_factors:
        worker = pool.spawn(_load_factor, secID, group, beginDate, endDate, username)
        workers.append(worker)

    result = {}
    if len(workers) > 0:
        pool.join()
        for worker in workers:
            val = worker.value
            if val[0] < 0:
                raise Exception(val[1])
            ast_data = val[1]
            for key in ast_data:
                result[key] = pd.DataFrame(ast_data[key])
    return pd.Panel(result)



# -*- coding: utf-8 -*-

import requests
import traceback
import os

env = os.getenv('env', 'prd')
gw_env = os.getenv('gw_env', 'prd')

if env == 'prd' or (env == 'local' and gw_env == 'prd'):
    server = ('10.22.220.82', 80, 'api.wmcloud.com', 443)
    URL_PREFIX = 'https://{}.wmcloud.com'
    EXT_MFHANDLER_HOST = 'https://gw.wmcloud.com/mercury_ops'
    PING_URL = "https://{}.wmcloud.com/mercury_community/w/ping"
else:
    server = ('10.22.220.42', 80, 'api.wmcloud-stg.com', 443)
    URL_PREFIX = 'https://{}.wmcloud-stg.com'
    EXT_MFHANDLER_HOST = 'https://gw.wmcloud-stg.com/mercury_ops'
    # EXT_MFHANDLER_HOST = 'http://10.20.233.4:8090'
    PING_URL = 'https://{}.wmcloud-stg.com/mercury_community/w/ping'


FOLDERS =  []
LOCAL_PATH = './'

### URL Config
URL = {}
URL_TEMPLATE = {}

### NOTEBOOK, DATA
update = URL_TEMPLATE.update(
    {'AUTHORIZE_URL': "/usermaster/authenticate/v1.json", 'TOKEN_URL': "/usermaster/oauth2/token.json?lang=zh",
     'MERCURY_URL': '/mercury/api/databooks', 'UQER_AUTH_URL': '/cloud/identity/uqer.json',
     'DATA_URL': '/mercury/databooks', 'NOTEBOOK_URL': '/mercury/api/notebooks?recursion',
     'NOTEBOOK_DELETE': '/mercury/api/notebooks/{}?force=1', 'DOWN_NOTEBOOK_URL': '/mercury/files',
     'SDK_URL': "/mercury_community", })

SDK_STATIC_URL = 'http://uqer.io/pro/download/sdk/uqer_sdk.zip'


### Live Trading
URL_TEMPLATE.update({
    'Live_Trading_URL' : "/mercury_trade/strategy",
    'Live_Order_URL' : "/mercury_trade/strategy/{}/order",
})

### User factor
URL_TEMPLATE.update({
    'Create_Factor': '/mercury_trade/customfactor',
    'Upload_FactorData': '/mercury_trade/customfactordata',
    'Get_Factor': '/mercury_trade/customfactor'
})

def check_gateway(gw=None):
    
    GATEWAY = ['gw', 'gw01', 'gw02', 'gw03']

    global URL, URL_TEMPLATE, URL_PREFIX, PING_URL
    ### 检查是否自定义网关
    if gw:
        good_gw = gw
    else:
        ### 网关检测
        good_gw = 'gw'
        for _gw in GATEWAY:
            _url = PING_URL.format(_gw)
            try:
                r = requests.get(_url)
                if r.ok:
                    good_gw = _gw
                    break
            except:
                print ('网关测试过程发生错误，错误信息如下：')
                traceback.print_exc()
                continue

    URL_PREFIX = URL_PREFIX.format(good_gw)

    ### 配置 URL
    for k,v in URL_TEMPLATE.items():
        URL[k] = URL_PREFIX + v

    return URL


URL = check_gateway()


def get_url(url_name):
    """利用此函数实现动态 url
    """
    global URL
    url = URL.get(url_name, '')

    if not url:
        print ('url 获取失败')
        print ('url 名字: {}'.format(url_name))
        print ('当前 url 池: {}'.format(url_name))
        raise ValueError('Error')
    else:
        return url

AUTHORIZE_URL = get_url('AUTHORIZE_URL')
TOKEN_URL = get_url('TOKEN_URL')
MERCURY_URL = get_url('MERCURY_URL')
UQER_AUTH_URL = get_url('UQER_AUTH_URL')
NOTEBOOK_URL = get_url('NOTEBOOK_URL') 
NOTEBOOK_DELETE = get_url('NOTEBOOK_DELETE') 
DATA_URL = get_url('DATA_URL') 
DOWN_NOTEBOOK_URL = get_url('DOWN_NOTEBOOK_URL') 
SDK_URL = get_url('SDK_URL')
Live_Trading_URL = get_url('Live_Trading_URL') 
Live_Order_URL = get_url('Live_Order_URL') 
Create_Factor = get_url('Create_Factor') 
Upload_FactorData = get_url('Upload_FactorData') 
Get_Factor = get_url('Get_Factor') 


### python 3.x unicode
try:
    unicode
except:
    unicode = str





#coding=utf-8

from . import settings
from .settings import global_cache_map
settings = settings.Settings()
from .DATAYES import *
from .OTHER import *


try:
    import __builtin__
except:
    import builtins as __builtin__

from . import DATAYES
from . import CCXE
from . import IT
from . import TEJ
from . import GG
from . import JY
from . import ACMR
from . import SW
from . import QAI
from . import JL
from . import CHINABOND
from . import IVOLATILITY
from . import FDC
from . import SILKRIVER
from . import QGW
from . import CSI
from . import JDW

from .api_base import retry_interval, max_retries
from . import ACMR
from . import IT
from . import JL
from . import JDW
from . import NH
from . import DYZH
from . import FDC
from . import FH
from . import SILKRIVER
from . import QGW
from . import DATAYES
from . import IVOLATILITY
from . import CHINABOND
from . import JYDB

from .api_base import retry_interval, max_retries


# -*- coding: UTF-8 -*-
from . import api_base
try:
    from StringIO import StringIO
except:
    from io import StringIO
import pandas as pd
import sys
from datetime import datetime
from .api_base import get_cache_key, get_data_from_cache, put_data_in_cache, pretty_traceback
import inspect
try:
    unicode
except:
    unicode = str

__doc__="恒生聚源"



# -*- coding: UTF-8 -*-
from . import api_base
try:
    from StringIO import StringIO
except:
    from io import StringIO
import pandas as pd
import sys
from datetime import datetime
from .api_base import get_cache_key, get_data_from_cache, put_data_in_cache, pretty_traceback
import inspect
try:
    unicode
except:
    unicode = str

__doc__="申万"
def __MktSWidxdGet(tradeDate, ticker = "", field = "", pandas = "1"):
    """
    获取一只申万指数，在历史某交易日的行情信息，包含当日开盘价，最高价，最低价，收盘价等
    
    :param tradeDate: 交易日期
    :param ticker: 指数代码,可以是列表,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/idxMarket/getMktSWidxd.csv?ispandas=1&') 
    try:
        tradeDate = tradeDate.strftime('%Y%m%d')
    except:
        tradeDate = tradeDate.replace('-', '')
    requestString.append("tradeDate=%s"%(tradeDate))
    requestString.append("&ticker=")
    if hasattr(ticker,'__iter__') and not isinstance(ticker, str):
        if len(ticker) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = ticker
            requestString.append(None)
        else:
            requestString.append(','.join(ticker))
    else:
        requestString.append(ticker)
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, '__MktSWidxdGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, '__MktSWidxdGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'ticker', u'secShortName', u'tradeDate', u'openIndex', u'lowIndex', u'highIndex', u'closeIndex', u'chg', u'chgPct', u'value', u'volume', u'PE', u'PB', u'negMarketValueA', u'marketValue']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'ticker': 'str','secShortName': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()




# -*- coding: UTF-8 -*-
from . import api_base
try:
    from StringIO import StringIO
except:
    from io import StringIO
import pandas as pd
import sys
from datetime import datetime
from .api_base import get_cache_key, get_data_from_cache, put_data_in_cache, pretty_traceback
import inspect
try:
    unicode
except:
    unicode = str

__doc__="IVolatility"
def DerIvGet(SecID, beginDate, endDate, optID = "", field = "", pandas = "1"):
    """
    原始隐含波动率,包括期权价格、累计成交量、持仓量、隐含波动率等。
    
    :param SecID: 证券展示代码
    :param beginDate: 起始日期，输入格式“YYYYMMDD”
    :param endDate: 截止日期，输入格式“YYYYMMDD”
    :param optID: 期权合约编码,可以是列表,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/IV/getDerIv.csv?ispandas=1&') 
    if not isinstance(SecID, str) and not isinstance(SecID, unicode):
        SecID = str(SecID)

    requestString.append("SecID=%s"%(SecID))
    try:
        beginDate = beginDate.strftime('%Y%m%d')
    except:
        beginDate = beginDate.replace('-', '')
    requestString.append("&beginDate=%s"%(beginDate))
    try:
        endDate = endDate.strftime('%Y%m%d')
    except:
        endDate = endDate.replace('-', '')
    requestString.append("&endDate=%s"%(endDate))
    requestString.append("&optID=")
    if hasattr(optID,'__iter__') and not isinstance(optID, str):
        if len(optID) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = optID
            requestString.append(None)
        else:
            requestString.append(','.join(optID))
    else:
        requestString.append(optID)
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, 'DerIvGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, 'DerIvGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'SecID', u'ticker', u'exchangeCD', u'secShortName', u'tradeDate', u'closePriceAdj', u'optID', u'expDate', u'strikePrice', u'contractType', u'sontractStyle', u'ask', u'bid', u'meanPrice', u'settlePrice', u'IV', u'turnoverVol', u'openInt', u'closePriceUnadj', u'forwardPrice', u'isInterPolated', u'delta', u'vega', u'gamma', u'theta', u'rho']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'SecID': 'str','ticker': 'str','exchangeCD': 'str','secShortName': 'str','optID': 'str','contractType': 'str','sontractStyle': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def DerIvHvGet(SecID, beginDate, endDate, period = "", field = "", pandas = "1"):
    """
    历史波动率，各个时间段的收盘－收盘历史波动率。
    
    :param SecID: 证券展示代码
    :param beginDate: 起始日期，输入格式“YYYYMMDD”
    :param endDate: 截止日期，输入格式“YYYYMMDD”
    :param period: 期限,可以是列表,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/IV/getDerIvHv.csv?ispandas=1&') 
    if not isinstance(SecID, str) and not isinstance(SecID, unicode):
        SecID = str(SecID)

    requestString.append("SecID=%s"%(SecID))
    try:
        beginDate = beginDate.strftime('%Y%m%d')
    except:
        beginDate = beginDate.replace('-', '')
    requestString.append("&beginDate=%s"%(beginDate))
    try:
        endDate = endDate.strftime('%Y%m%d')
    except:
        endDate = endDate.replace('-', '')
    requestString.append("&endDate=%s"%(endDate))
    requestString.append("&period=")
    if hasattr(period,'__iter__') and not isinstance(period, str):
        if len(period) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = period
            requestString.append(None)
        else:
            requestString.append(','.join(period))
    else:
        requestString.append(period)
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, 'DerIvHvGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, 'DerIvHvGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'SecID', u'ticker', u'exchangeCD', u'tradeDate', u'closePriceUnadj', u'period', u'hv']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'SecID': 'str','ticker': 'str','exchangeCD': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def DerIvIndexGet(SecID, beginDate, endDate, period = "", field = "", pandas = "1"):
    """
    隐含波动率指数，衡量30天至1080天到期平价期权的平均波动性的主要方法。
    
    :param SecID: 证券展示代码
    :param beginDate: 起始日期，输入格式“YYYYMMDD”
    :param endDate: 截止日期，输入格式“YYYYMMDD”
    :param period: 期限,可以是列表,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/IV/getDerIvIndex.csv?ispandas=1&') 
    if not isinstance(SecID, str) and not isinstance(SecID, unicode):
        SecID = str(SecID)

    requestString.append("SecID=%s"%(SecID))
    try:
        beginDate = beginDate.strftime('%Y%m%d')
    except:
        beginDate = beginDate.replace('-', '')
    requestString.append("&beginDate=%s"%(beginDate))
    try:
        endDate = endDate.strftime('%Y%m%d')
    except:
        endDate = endDate.replace('-', '')
    requestString.append("&endDate=%s"%(endDate))
    requestString.append("&period=")
    if hasattr(period,'__iter__') and not isinstance(period, str):
        if len(period) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = period
            requestString.append(None)
        else:
            requestString.append(','.join(period))
    else:
        requestString.append(period)
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, 'DerIvIndexGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, 'DerIvIndexGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'SecID', u'ticker', u'exchangeCD', u'tradeDate', u'closePriceAdj', u'period', u'ivIndexCall', u'ivIndexPut', u'ivIndexMean']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'SecID': 'str','ticker': 'str','exchangeCD': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def DerIvIvpDeltaGet(SecID, beginDate, endDate, period = "", delta = "", field = "", pandas = "1"):
    """
    隐含波动率曲面(基于参数平滑曲线)，基于delta（0.1至0.9,0.05升步）和到期日（1个月至3年）而标准化的曲面。
    
    :param SecID: 证券展示代码
    :param beginDate: 起始日期，输入格式“YYYYMMDD”
    :param endDate: 截止日期，输入格式“YYYYMMDD”
    :param period: 期限,可以是列表,可空
    :param delta: 德尔塔,可以是列表,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/IV/getDerIvIvpDelta.csv?ispandas=1&') 
    if not isinstance(SecID, str) and not isinstance(SecID, unicode):
        SecID = str(SecID)

    requestString.append("SecID=%s"%(SecID))
    try:
        beginDate = beginDate.strftime('%Y%m%d')
    except:
        beginDate = beginDate.replace('-', '')
    requestString.append("&beginDate=%s"%(beginDate))
    try:
        endDate = endDate.strftime('%Y%m%d')
    except:
        endDate = endDate.replace('-', '')
    requestString.append("&endDate=%s"%(endDate))
    requestString.append("&period=")
    if hasattr(period,'__iter__') and not isinstance(period, str):
        if len(period) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = period
            requestString.append(None)
        else:
            requestString.append(','.join(period))
    else:
        requestString.append(period)
    requestString.append("&delta=")
    if hasattr(delta,'__iter__') and not isinstance(delta, str):
        if len(delta) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = delta
            requestString.append(None)
        else:
            requestString.append(','.join(delta))
    else:
        requestString.append(delta)
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, 'DerIvIvpDeltaGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, 'DerIvIvpDeltaGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'SecID', u'ticker', u'exchangeCD', u'tradeDate', u'closePriceUnadj', u'period', u'delta', u'moneyness', u'strikePrice', u'IV']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'SecID': 'str','ticker': 'str','exchangeCD': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def DerIvParamGet(SecID, beginDate, endDate, expDate = "", field = "", pandas = "1"):
    """
    隐含波动率参数化曲面，由二阶方程波动曲线在每个到期日平滑后的曲面（a,b,c曲线系数）
    
    :param SecID: 证券展示代码
    :param beginDate: 起始日期，输入格式“YYYYMMDD”
    :param endDate: 截止日期，输入格式“YYYYMMDD”
    :param expDate: 到期日期,可以是列表,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/IV/getDerIvParam.csv?ispandas=1&') 
    if not isinstance(SecID, str) and not isinstance(SecID, unicode):
        SecID = str(SecID)

    requestString.append("SecID=%s"%(SecID))
    try:
        beginDate = beginDate.strftime('%Y%m%d')
    except:
        beginDate = beginDate.replace('-', '')
    requestString.append("&beginDate=%s"%(beginDate))
    try:
        endDate = endDate.strftime('%Y%m%d')
    except:
        endDate = endDate.replace('-', '')
    requestString.append("&endDate=%s"%(endDate))
    requestString.append("&expDate=")
    if hasattr(expDate,'__iter__') and not isinstance(expDate, str):
        if len(expDate) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = expDate
            requestString.append(None)
        else:
            requestString.append(','.join(expDate))
    else:
        requestString.append(expDate)
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, 'DerIvParamGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, 'DerIvParamGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'SecID', u'ticker', u'exchangeCD', u'expDate', u'tradeDate', u'closePriceUnadj', u'forwardPrice', u'parabolaA', u'parabolaB', u'parabolaC']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'SecID': 'str','ticker': 'str','exchangeCD': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def DerIvRawDeltaGet(SecID, beginDate, endDate, period = "", delta = "", field = "", pandas = "1"):
    """
    隐含波动率曲面(基于原始隐含波动率)，基于delta（0.1至0.9,0.05升步）和到期日（1个月至3年）而标准化的曲面。
    
    :param SecID: 证券展示代码
    :param beginDate: 起始日期，输入格式“YYYYMMDD”
    :param endDate: 截止日期，输入格式“YYYYMMDD”
    :param period: 期限,可以是列表,可空
    :param delta: 德尔塔,可以是列表,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/IV/getDerIvRawDelta.csv?ispandas=1&') 
    if not isinstance(SecID, str) and not isinstance(SecID, unicode):
        SecID = str(SecID)

    requestString.append("SecID=%s"%(SecID))
    try:
        beginDate = beginDate.strftime('%Y%m%d')
    except:
        beginDate = beginDate.replace('-', '')
    requestString.append("&beginDate=%s"%(beginDate))
    try:
        endDate = endDate.strftime('%Y%m%d')
    except:
        endDate = endDate.replace('-', '')
    requestString.append("&endDate=%s"%(endDate))
    requestString.append("&period=")
    if hasattr(period,'__iter__') and not isinstance(period, str):
        if len(period) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = period
            requestString.append(None)
        else:
            requestString.append(','.join(period))
    else:
        requestString.append(period)
    requestString.append("&delta=")
    if hasattr(delta,'__iter__') and not isinstance(delta, str):
        if len(delta) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = delta
            requestString.append(None)
        else:
            requestString.append(','.join(delta))
    else:
        requestString.append(delta)
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, 'DerIvRawDeltaGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, 'DerIvRawDeltaGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'SecID', u'ticker', u'tradeDate', u'closePriceUnadj', u'period', u'delta', u'moneyness', u'strikePrice', u'IV']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'SecID': 'str','ticker': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def DerIvSurfaceGet(SecID, beginDate, endDate, contractType = "", field = "", pandas = "1"):
    """
    隐含波动率曲面(在值程度)，基于在值程度而标准化的曲面。执行价格区间在-60%到+60%，5%升步，到期区间为1个月至3年。
    
    :param SecID: 证券展示代码
    :param beginDate: 起始日期，输入格式“YYYYMMDD”
    :param endDate: 截止日期，输入格式“YYYYMMDD”
    :param contractType: 期权类型,C代表认购，P代表认沽,可以是列表,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/IV/getDerIvSurface.csv?ispandas=1&') 
    if not isinstance(SecID, str) and not isinstance(SecID, unicode):
        SecID = str(SecID)

    requestString.append("SecID=%s"%(SecID))
    try:
        beginDate = beginDate.strftime('%Y%m%d')
    except:
        beginDate = beginDate.replace('-', '')
    requestString.append("&beginDate=%s"%(beginDate))
    try:
        endDate = endDate.strftime('%Y%m%d')
    except:
        endDate = endDate.replace('-', '')
    requestString.append("&endDate=%s"%(endDate))
    requestString.append("&contractType=")
    if hasattr(contractType,'__iter__') and not isinstance(contractType, str):
        if len(contractType) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = contractType
            requestString.append(None)
        else:
            requestString.append(','.join(contractType))
    else:
        requestString.append(contractType)
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, 'DerIvSurfaceGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, 'DerIvSurfaceGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'SecID', u'ticker', u'tradeDate', u'closePriceUnadj', u'period', u'strike', u'outOfMoney', u'contractType', u'IV', u'delta']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'SecID': 'str','ticker': 'str','contractType': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()




#coding=utf-8

import traceback
from datetime import date
import numpy as np
import pandas as pd
import json
import math

from ..config import *


class MFUtilityError(Exception):
    def __init__(self, log_info):
        self.log_info = log_info

    def __str__(self):
        return str(self.log_info)


class MFInputError(MFUtilityError):
    pass


class InputLengthError(MFInputError):
    pass


class InputTypeError(MFInputError):
    pass


class InputConfigError(MFInputError):
    pass


class MFOutputError(MFUtilityError):
    pass


class MFNetworkError(MFUtilityError):
    pass


def raw_data_check(raw_data):
    if not isinstance(raw_data, (dict, pd.Series)):
        raise InputTypeError('Input raw_data is not dictionary or Series')
    if len(raw_data) <= 1:
        # print 'raw data length is ', len(raw_data)
        raise InputLengthError('Input raw_data length too short')
    
    if isinstance(raw_data, dict):
        items = raw_data.items
    elif isinstance(raw_data, pd.Series):
        items = raw_data.iteritems

    for key, value in items():
        if not isinstance(value, (int, float, np.int64)):
            raise InputTypeError('Input raw_data value is not value')
        if isinstance(key, unicode):
            key = str(key)
        if not isinstance(key, (str, unicode)):
            raise InputTypeError('Input raw_data key is not str')
        key = str(key)
        key_check(key)


def key_check(input_str):
    if len(input_str) != 6 and len(input_str) != 11:
        raise InputTypeError('Input raw_data key length error')
    if len(input_str) == 11:
        key = input_str[:6]
        end = input_str[6:]
        if end not in ['.XSHE', '.XSHG']:
            raise InputTypeError('Input raw_data key type error')
    else:
        key = input_str[:6]

    if not key.isdigit():
        raise InputTypeError('Input raw_data key type error')


def industry_check(industry_type):
    if industry_type not in ['long', 'short', 'day', 'SW1', 'SW2']:
        raise InputConfigError('Industry Type Config Error')


def risk_model_check(risk_model):
    if risk_model not in ['long', 'short', 'day']:
        raise InputConfigError('Risk Model Config Error')


def universe_check(universe):
    '''
    :param universe: universe type
    :return: mapped universe type
    '''
    map_dict = {'HS300': 'HSSLL', 'SH50': 'SZWL', 'SH180': 'SZYBL', 'ZZ500': 'ZZWLL'}
    if universe in map_dict:
        universe = map_dict[universe]
    if universe not in ['HSSLL', 'SZWL', 'SZYBL', 'SZZS', 'ZZWLL']:
        raise InputConfigError('Universe Config Error')
    return universe


def date_check(target_date):
    if not isinstance(target_date, str) and not isinstance(target_date, unicode):
        raise InputTypeError('Input Target Date Type Error')
    if len(target_date) not in [8, 10]:
        raise InputTypeError('Input Target Data Length Error')
    if date.today().strftime('%Y%m%d') <= target_date.replace('-', ''):
        raise InputTypeError('Input Target Date Config Error')


def risk_factor_check(exclude_factor_list):
    if not isinstance(exclude_factor_list, list):
        raise InputTypeError('Input Exclude Factor List Type Error')
    for factor in exclude_factor_list:
        if factor not in ['BETA', 'RESVOL', 'MOMENTUM', 'SIZE', 'SIZENL', 'EARNYILD', 'BTOP', 'GROWTH', 'LEVERAGE',
                          'LIQUIDTY']:
            raise InputConfigError('Input Risk Factor Config Error')


def isnull(var):
    '''
    @summary: check the variable @var is None or empty array/timeseries/dataframe or not
    @var: input variable
    @return: bool
    '''
    if var is None:
        return True
    elif var is np.nan:
        return True
    elif isinstance(var, np.ndarray) and var.size == 0:
        return True
    elif isinstance(var, (pd.Series, pd.Index)) and var.size == 0:
        return True
    elif isinstance(var, pd.DataFrame) and (var.shape[0] == 0 or var.shape[1] == 0):
        return True
    else:
        return False


def handle_recv_json(requests_return):
    '''
    :param requests_return: JSON received from server
    :param flag:
    :param record_nan: the dict record Nan value
    :return:
    '''
    try:
        input_dict = requests_return.json()
    except:
        raise MFOutputError('JSON decoder error')

    if input_dict.get('result').get('status'):
        res_dict = input_dict.get('data')
        if not isinstance(res_dict, dict):
            res_dict = json.loads(input_dict.get('data'))
        new_dict = {}
        for k, v in res_dict.items():
            new_dict[str(k)] = v
        return new_dict
    else:
        raise MFOutputError(input_dict.get('result').get('error'))










# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import unicode_literals

import sys
import os
import dateutil.parser as dt_parser
from .config import *


def convert_date(date, format='%Y-%m-%d'):
    try:
        if isinstance(date, (str, unicode)):
            date = dt_parser.parse(date)
    except Exception as e:
        raise Exception('date:{}格式不能识别。'%date)

    return date.strftime(format)
    






# coding=utf-8

import json
import requests
import numpy as np
from . signal_process_check import *

from ..config import *
from ..uqer import session


def long_only(raw_data, select_type, top_ratio, weight_type, target_date, risk_model='short', industry_type='SW1',
              universe_type='HS300'):
    '''
    只做多组合构建函数

    :param raw_data: [Dict/Series] 输入的因子值，key为ticker/secID，value为因子值
    :param select_type: [int] 行业类型选择，包括分行业(1)和全行业(0)，若选择分行业，则以后所有选股和配权操作均在行业内部计算
    :param top_ratio: [float] 选择比例，合法输入为(0,0.5)区间的浮点数，表示选取前top_ratio*100%的股票
    :param weight_type: [int] 初始权重类型，包括等权重(0), 流通市值权重(1), 风险权重(2), 因子权重(3)，合法输入为[0,3]区间的整数
    :param target_date: [String] 基准日期，合理输入为调仓前一个交易日，日期格式为'YYYYMMDD'
    :param risk_model: [String] 通联风险模型，包括短期、长期和日度，分别用short/long/day表示；当weight_type=2时，可通过该参数选择风险模型类型
    :param industry_type: [String] 行业分类选择, 包括申万一级、申万二级, 分别用SW1/SW2表示，默认为SW1
    当select_type=1时，可通过改变该参数调整行业分类选择
    :param universe_type: [String] 基准选择，包括沪深300、中证500、中证700、中证800，分别用HS300/ZZ500/ZZ700/ZZ800表示, 默认为HS300
    当select_type=1时，可通过改变该参数调整基准选择
    :return: [Dict/Series] 组合构建的结果，key值为ticker/secID, value为组合中的价值权重
    '''

    # Check Input Data
    is_series = False

    if isinstance(raw_data, dict):
        items = raw_data.items
    elif isinstance(raw_data, pd.Series):
        items = raw_data.iteritems
        is_series = True

    raw_data_check(raw_data)
    date_check(target_date)
    industry_check(industry_type)
    universe_type = universe_check(universe_type)

    if risk_model in ['long', 'short', 'day']:
        risk_model = 'datayes_' + risk_model

    if select_type not in [0, 1]:
        raise InputConfigError("select_type config error")

    if weight_type not in range(4):
        raise InputConfigError("weight_type config error")

    if not isinstance(top_ratio, (float, int, np.int64)):
        raise InputTypeError('top_ratio type error')

    if not 0 < top_ratio < 0.5:
        raise InputConfigError("top_ratio config error, should in (0, 0.5)")

    # Ticker to SecID
    isTicker = False
    for k, v in items():
        if len(k) == 11:
            isTicker = True
            break
    record_ticker = {}
    if isTicker:
        for k, v in items():
            record_ticker[k[:6]] = k

    # Deal with Nan
    record_nan = {}
    input_dict = {}
    for k, v in items():
        if isinstance(v, (int, float)) and math.isnan(v):
            record_nan[k[:6]] = v
        elif v == np.inf or v == -np.inf:
            record_nan[k[:6]] = v
        elif isinstance(v, np.generic):
            input_dict[k[:6]] = np.asscalar(v)
        else:
            input_dict[k[:6]] = v

    send_data = {}
    config_dict = {}
    config_dict['select_type'] = select_type
    config_dict['top_ratio'] = top_ratio
    config_dict['weight_type'] = weight_type
    config_dict['date'] = target_date.replace('-', '')
    config_dict['industry_type'] = industry_type
    config_dict['universe_type'] = universe_type
    config_dict['risk_model'] = risk_model
    send_data['raw_data'] = input_dict
    send_data['config'] = config_dict
    send_data['type'] = 'long_only'
    try:
        r = session.post(EXT_MFHANDLER_HOST + '/get_portfolio', json=send_data)
    except:
        raise MFNetworkError('long_only Network Error')
    ret = handle_recv_json(r)
    # Deal with Nan, Add Nan to result
    # if record_nan:
    #     for k, v in record_nan.items():
    #         ret[k] = v

    # Ticker to SecID, change the key of dict
    final_result = {}
    if isTicker:
        for k, v in ret.items():
            final_result[record_ticker[k]] = v
    else:
        final_result = ret

    if is_series:
        return pd.Series(final_result)
    else:
        return final_result


def simple_long_only(raw_data, target_date, industry_type='SW1', universe_type='HSSLL', **kwargs):
    '''
    简单只做多组合构建函数

    :param raw_data: [Dict/Series] 输入的因子值，key为ticker/secID，value为因子值
    :param target_date: [String] 行业中性的基准日期，合理输入为调仓前一个交易日，日期格式为'YYYYMMDD'
    :param industry_type: [String] 行业分类选择, 包括申万一级、申万二级, 分别用SW1/SW2，默认为SW1
    :param universe_type: [String] 基准选择，包括沪深300、中证500、中证700、中证800，分别用HS300/ZZ500/ZZ700/ZZ800表示, 默认为HS300
    :param holding_weight: [Dict] 输入的持仓权重数据，key值为ticker，value值为现有持仓组合中的价值权重
    :return: [Dict/Series] 组合构建的结果，key值为ticker, value为组合中的价值权重
    '''
    # Check Input Data
    is_series = False
    if isinstance(raw_data, dict):
        items = raw_data.items
    elif isinstance(raw_data, pd.Series):
        items = raw_data.iteritems
        is_series = True

    raw_data_check(raw_data)
    industry_check(industry_type)
    universe_type = universe_check(universe_type)
    date_check(target_date)
    # 历史字段的支持
    if 'holding_weight' in kwargs:
        print ("holding_weight参数已经不再使用了，请参考帮助文档")

    # Ticker to SecID
    isTicker = False
    for k, v in items():
        if len(k) == 11:
            isTicker = True
            break
    record_ticker = {}
    if isTicker:
        for k, v in items():
            record_ticker[k[:6]] = k

    # Deal with Nan
    record_nan = {}
    input_dict = {}
    for k, v in items():
        if isinstance(v, (int, float)) and math.isnan(v):
            record_nan[k[:6]] = v
        elif v == np.inf or v == -np.inf:
            record_nan[k[:6]] = v
        elif isinstance(v, np.generic):
            input_dict[k[:6]] = np.asscalar(v)
        else:
            input_dict[k[:6]] = v

    send_data = {}
    config_dict = {}
    config_dict['date'] = target_date.replace('-', '')
    config_dict['industry_type'] = industry_type
    config_dict['universe_type'] = universe_type
    send_data['raw_data'] = input_dict
    send_data['config'] = config_dict

    send_data['type'] = 'simple_long_only'
    try:
        r = session.post(EXT_MFHANDLER_HOST + '/get_portfolio', json=send_data)
    except:
        raise MFNetworkError('simple_long_only Network Error')
    ret = handle_recv_json(r)

    # Deal with Nan, Add Nan to result
    # if record_nan:
    #     for k, v in record_nan.items():
    #         ret[k] = v

    # Ticker to SecID, change the key of dict
    final_result = {}
    if isTicker:
        for k, v in ret.items():
            final_result[record_ticker[k]] = v
    else:
        final_result = ret

    if is_series:
        return pd.Series(final_result)
    else:
        return final_result



# -*- coding: utf-8 -*-
# 通联数据机密
# --------------------------------------------------------------------
# 通联数据股份公司版权所有 © 2013-2019
#
# 注意：本文所载所有信息均属于通联数据股份公司资产。本文所包含的知识和技术概念均属于
# 通联数据产权，并可能由中国、美国和其他国家专利或申请中的专利所覆盖，并受商业秘密或
# 版权法保护。
# 除非事先获得通联数据股份公司书面许可，严禁传播文中信息或复制本材料。
#
# DataYes CONFIDENTIAL
# --------------------------------------------------------------------
# Copyright © 2013-2019 DataYes, All Rights Reserved.
#
# NOTICE: All information contained herein is the property of DataYes
# Incorporated. The intellectual and technical concepts contained herein are
# proprietary to DataYes Incorporated, and may be covered by China, U.S. and
# Other Countries Patents, patents in process, and are protected by trade
# secret or copyright law.
# Dissemination of this information or reproduction of this material is
# strictly forbidden unless prior written permission is obtained from DataYes.
__version__ = '1.0.0'



# -*- coding: utf-8 -*-
# 通联数据机密
# --------------------------------------------------------------------
# 通联数据股份公司版权所有 © 2013-2017
#
# 注意：本文所载所有信息均属于通联数据股份公司资产。本文所包含的知识和技术概念均属于
# 通联数据产权，并可能由中国、美国和其他国家专利或申请中的专利所覆盖，并受商业秘密或
# 版权法保护。
# 除非事先获得通联数据股份公司书面许可，严禁传播文中信息或复制本材料。
#
# DataYes CONFIDENTIAL
# --------------------------------------------------------------------
# Copyright © 2013-2016 DataYes, All Rights Reserved.
#
# NOTICE: All information contained herein is the property of DataYes
# Incorporated. The intellectual and technical concepts contained herein are
# proprietary to DataYes Incorporated, and may be covered by China, U.S. and
# Other Countries Patents, patents in process, and are protected by trade
# secret or copyright law.
# Dissemination of this information or reproduction of this material is
# strictly forbidden unless prior written permission is obtained from DataYes.
'''
Created on 2017年4月28日

@author: jiangtao.sheng
'''

from . data_service.market_service import MarketService
from . data_service.asset_service import AssetService
from . data_service.tick_roller import TickRoller
from . data_service.utils.datetime_utils import get_trading_days
from . data_service.loader.factor_loader import translate_factors
from . data_service.loader.cache_api import get_updated_date, HelloUser, get_td_from_cache
from . api_base import is_pro_user, is_enterprise_user
import datetime
import pandas as pd
import uuid
import time
import os
import numpy as np
from dateutil.parser import parse
from . data_service.const import (EQUITY_DAILY_FIELDS,
                                  STOCK_DAILY_FQ_FIELDS,
                                  EQUITY_MINUTE_FIELDS,
                                  FUTURES_DAILY_FIELDS,
                                  FUTURES_MINUTE_FIELDS,
                                  FS_FIELD,
                                  ALIAS_DAILY_PRICE, ALIAS_MINUTE_PRICE)
MAX_END_DATE = '3000'
MAX_MINUTE_LENGTH = 10000000


MKT_DAILY_FIELDS = set(EQUITY_DAILY_FIELDS) | set(STOCK_DAILY_FQ_FIELDS) \
    | set(FUTURES_DAILY_FIELDS) | set(ALIAS_DAILY_PRICE)

MKT_INTRADAY_FIELDS = set(EQUITY_MINUTE_FIELDS) | set(FUTURES_MINUTE_FIELDS) | \
    set(ALIAS_MINUTE_PRICE)

price_key_mapping = {}


def uniform_date(input_date):
    if isinstance(input_date, basestring):
        format_date = None
        try:
            format_date = parse(input_date)
        except:
            print (u'输入的日期：%s 无法解析，请重新输入' % input_date)
        return format_date
    if isinstance(input_date, datetime.datetime) or isinstance(input_date, datetime.date):
        return input_date
    else:
        print (u'输入的日期格式不支持，请使用 datetime或string类型.')
        return


TODAY = datetime.datetime.today()
TODAY_STR = TODAY.strftime('%Y%m%d')
YESTERDAY = TODAY - datetime.timedelta(days=1)


def get_data_cube(symbol, field, start, end=None, freq='1d', style='sat', adj=None, **kwargs):
    '''
        :param symbol: [list or str] 支持股票、基金、期货（包括连续合约，主力合约、次主力合约）、期权、指数，股票基金。
        :param field: [list or str] 支持行情数据、424因子数据、众筹因子
        :param start： [str or datetime] 返回数据的开始日期，支持"2017-02-02"的字符串格式，也支持Python 中时间和日期格式。
        :param end： [str or datetime] 默认值为当天，返回数据的截止日期，支持"2017-02-02"的字符串格式，也支持Python 中时间和日期格式
        :param freq: [str] 默认值为d。支持日(1d)，多频率的分钟线(1m,5m,15m,30m,60m)
        :param adj: [str] 复权方式 默认为空值，'pre'指代前复权 
        :param style： [str] 数据返回的类型，默认值sat。支持ast/sat/tas，分钟线不支持'tas'。
                                                                其中'a'表示'attribute'、's'表示symbol、't'表示时间。
                        如'ast'就表示返回的Panel中的键是attribute，其值为列为symbol、行为time的DataFrame

    '''
    entry_time = time.time()
    uid = str(uuid.uuid1())
    # 权限控制
    if not (is_pro_user() or is_enterprise_user()):
        print (u"该API仅限优矿专业版、企业版使用，请联系优矿客服 service.uqer@datayes.com，谢谢！")
        return

    # format symbol
    symbol = __format_str_to_arr(symbol, "symbol")
    if not symbol:
        return

    # format field
    field = __format_str_to_arr(field, "field")
    if not field:
        return

    # validate freq
    if isinstance(freq, basestring):
        if freq not in ['1d', '1m', 'd', 'm', '5m', '15m', '30m', '60m']:
            print (
                u"目前只支持'1d','1m','5m', '15m', '30m', '60m'六个频率类型，暂不支持:%s。" % freq)
            return
    else:
        print (u"参数freq的值支持str类型，请校验后重新输入。")

    # validate adj
    if adj is not None and adj != 'pre':
        print (u"目前adj参数只支持'pre'，表示行情前复权，无法识别传入的adj参数值，默认返回未复权行情数据。")

    # parse date
    start_date = uniform_date(start)
    if start_date is None:
        return

    stocks = AssetService.findout_stocks(symbol)
    funds = AssetService.findout_funds(symbol)
    index = AssetService.findout_indexes(symbol)
    futures = AssetService.findout_futures(symbol)

    # 分钟线限制
    if freq in ['1m', '5m', '15m', '30m', '60m', 'm']:
        if (stocks or funds or index) and start_date < datetime.datetime(2009, 1, 1):
            print (
                u"使用分钟线：目前股票、基金、指数支持的最早start为2009-01-01，期货支持的最早start为2010-01-01，自动修改为2009-01-01。")
            start_date = datetime.datetime(2009, 1, 1)
        elif start_date < datetime.datetime(2010, 1, 1):
            print (u"使用分钟线：期货支持的最早start为2010-01-01，自动修改为2010-01-01。")
            start_date = datetime.datetime(2010, 1, 1)

    # validate field
    mkt_daily_fields = set(field) & MKT_DAILY_FIELDS
    mkt_minute_fields = (set(field) & MKT_INTRADAY_FIELDS)
    mkt_fields = mkt_daily_fields | mkt_minute_fields
    if set(field) - set(mkt_fields):
        factors = translate_factors(
            list(set(field) - set(mkt_fields)), start_date.strftime('%Y%m%d'), **kwargs)
    else:
        factors = []
    fs_fields = list(set(field) & set(FS_FIELD))
    rest = set(field) - set(mkt_fields) - set(factors) - set(fs_fields)
    if rest:
        print (u"传入的参数fields中有 %s无法识别，请校验后重试。" % list(rest))

    lastest_stock_tdate = TODAY
    lastest_future_tdate = TODAY

    # parse end date
    # 用来智能判断今日行情是否到位，未到位则没有今天的这一行
    if end is None:
        data_due_date = os.getenv('data_due_date')
        if data_due_date:
            end_date = uniform_date(data_due_date)
        else:
            end_date = uniform_date(TODAY)
    else:
        end_date = uniform_date(end)
    if end_date is None:
        return

    if end_date >= uniform_date(TODAY) and get_trading_days(TODAY_STR, TODAY_STR):

        if stocks or funds or index:
            if freq in ['d', '1d']:
                lastest_stock_tdate = get_updated_date('daily_adj')
            elif freq in ['1m', '5m', '15m', '30m', '60m', 'm']:
                lastest_stock_tdate = get_updated_date('min_data')
        if futures:
            lastest_future_tdate = get_updated_date('future')
        if min([lastest_stock_tdate.strftime('%Y%m%d'), lastest_future_tdate.strftime('%Y%m%d')]) < TODAY_STR:
            print (u"今日行情还没有入库，默认获取截止到上一个交易日的数据。")
            end_date = uniform_date(YESTERDAY)

    if end_date < start_date:
        print (u"传入的start晚于end时间，请修正后重试。")
        return
    trading_days = [datetime.datetime.strptime(x, '%Y%m%d')
                    for x in get_td_from_cache(start_date.strftime('%Y%m%d'),
                                               end_date.strftime('%Y%m%d'))]
    if len(trading_days) == 0:
        print (u"传入的日期区间 start:%s, end:%s 不包含交易日，无数据返回" % (start, end))
        return
    start_date_str = start_date.strftime('%Y%m%d')
    end_date_str = end_date.strftime('%Y%m%d')

    # validate symbol
    stock_fund_tdates_dict = AssetService.get_stock_fund_tdates(stocks + funds)
    filtered_stock_fund = []
    # 删除股票和基金中间不在交易区间的
    for univ in stocks + funds:
        if univ in stock_fund_tdates_dict:
            for one_tuple in stock_fund_tdates_dict[univ]:
                if str(one_tuple[0]) <= end_date_str and \
                        (one_tuple[1] is None or str(one_tuple[1]) > start_date_str):
                    filtered_stock_fund.append(univ)
                    break
    # 对于股票和基金没有就是无法交易
#         else:
#             filtered_stock_fund.append(univ)
    if len(set(stocks) - set(filtered_stock_fund)) > 0:
        print (u"传入的股票列表中：%s在给定的时间段内没有交易，数据不会出现在结果中。" %
               list(set(stocks) - set(filtered_stock_fund)))
    if len(set(funds) - set(filtered_stock_fund)) > 0:
        print (u"传入的基金列表中：%s在给定的时间段内没有交易，数据不会出现在结果中。" %
               list(set(funds) - set(filtered_stock_fund)))

    filtered_futures = []
    if futures:
        future_tdates_dict = AssetService.get_future_tdates(futures)
        # 删除futures中间不在交易区间的期货

        for future in futures:
            if future in future_tdates_dict:
                if str(future_tdates_dict[future][0]) < end_date_str and \
                        str(future_tdates_dict[future][1]) > start_date_str:
                    filtered_futures.append(future)
            else:
                # 比如连续合约暂时查不到
                filtered_futures.append(future)

        if len(set(futures) - set(filtered_futures)) > 0:
            print ("传入的期货列表中：%s在给定的时间段内没有交易，数据不会出现在结果中。" %
                   list(set(futures) - set(filtered_futures)))

    # 去掉不合法的symbol
    invalid_symbol = list(
        set(symbol) - (set(stocks) | set(futures) | set(index) | set(funds)))
    if invalid_symbol:
        print ("无法识别传入的symbol参数中的%s，请检查后重试。" % invalid_symbol)

    # 整理最后合法的symbol
    symbol = list(
        set(filtered_stock_fund) | set(filtered_futures) | set(index))
    if len(symbol) == 0:
        return pd.Panel()

    HelloUser({"symbol": symbol, 'field': field, 'start': start_date.strftime('%Y%m%d'),
               'end': end_date.strftime('%Y%m%d'), 'freq': freq,
               'style': style, 'adj': adj, 'uid': uid})
    if freq in ['d', '1d']:

        market_service = MarketService.create_with_service(symbol,
                                                           mkt_daily_fields,
                                                           mkt_minute_fields,
                                                           factors,
                                                           fs_fields,
                                                           adj=True if adj == 'pre' else False,
                                                           **kwargs)
        market_service.batch_load_daily_data(trading_days)
        data = market_service.slice(symbols=symbol, fields=field,
                                    end_date=end_date, freq='d', start_date=start_date,
                                    style=style, rtype='frame', time_range=None)
        HelloUser({"symbol": symbol, 'field': field, 'start': start_date.strftime('%Y%m%d'),
                   'end': end_date.strftime('%Y%m%d'), 'freq': freq,
                   'style': style, 'adj': adj, 'uid': uid, 'cost': time.time() - entry_time})
        return pd.Panel(data).replace([None], np.nan)
    if freq in ['1m', '5m', '15m', '30m', '60m', 'm']:
        if 'IFZ0' in symbol:
            print ("IFZ0暂不支持分钟线数据。")
            symbol.remove('IFZ0')

        mkt_daily_fields = list(set(mkt_daily_fields) - set(mkt_minute_fields))

        if mkt_daily_fields:
            print ("传入字段：%s为日线字段，暂时没有分钟相关的数据。" % mkt_daily_fields)
        market_service = MarketService.create_with_service(symbol,
                                                           [],
                                                           mkt_minute_fields,
                                                           factors,
                                                           fs_fields,
                                                           adj=True if adj == 'pre' else False,
                                                           **kwargs)
        market_service.batch_load_daily_data(trading_days)
        market_service.rolling_load_minute_data(trading_days, None, freq)
        tick_roller = TickRoller(market_service)
        data_min = tick_roller.slice(prepare_dates=trading_days, end_time=MAX_END_DATE,
                                     time_range=MAX_MINUTE_LENGTH,
                                     fields=field, symbols=symbol, style=style,
                                     rtype='frame')
        HelloUser({"symbol": symbol, 'field': field, 'start': start_date.strftime('%Y%m%d'),
                   'end': end_date.strftime('%Y%m%d'), 'freq': freq,
                   'style': style, 'adj': adj, 'uid': uid, 'cost': time.time() - entry_time})

        return pd.Panel(data_min).replace([None], np.nan)


def __format_str_to_arr(param, param_name):
    if isinstance(param, basestring):
        if ',' in param:
            param = param.split(',')
        elif ';' in param:
            param = param.split(';')
        else:
            param = [param]
    elif isinstance(param, list):
        for item in param:
            if not isinstance(item, basestring):
                print (u"参数%s的值支持list of str或者str类型，请校验后重新输入。" % param_name)
                return []
    return [item.strip() for item in param if isinstance(item, basestring)]



# -*- coding: utf-8 -*-
import io
import sys

NAME = 'uqer'
MAINTAINER = 'uqer'
EMAIL = 'service.uqer@datayes.com'
URL = 'uqer.io'
LICENSE = 'DataYes'


import os

from setuptools import find_packages, setup

exec(compile(open('uqer/version.py').read(), 'pypkg/version.py', 'exec'))
version = __version__

setup(
      name = NAME,
      description = 'Package for DataYes Uqer API access',
      version = version,
      maintainer = MAINTAINER,
      maintainer_email = EMAIL,
      license = LICENSE,
      author = NAME,
      author_email = EMAIL,
      packages=find_packages(),
      package_data = {'': ['*.ipynb']},
      # install_requires = [
      #   "requests>=2.7.0",
      #   "pandas"
      # ]
)

if len(sys.argv)>=2 and sys.argv[1] in ['sdist']:
    os.system("mv dist/uqer-%s.zip dist/uqer.zip" %version)




# -*- coding: UTF-8 -*-
from . import api_base
try:
    from StringIO import StringIO
except:
    from io import StringIO
import pandas as pd
import sys
from datetime import datetime
from .api_base import get_cache_key, get_data_from_cache, put_data_in_cache, pretty_traceback
import inspect
try:
    unicode
except:
    unicode = str

__doc__="中汉指数"
def MktIdxdZHGet(tradeDate = "", beginDate = "", endDate = "", secID = "", ticker = "", field = "", pandas = "1"):
    """
    获取中汉指数日度行情数据，包含指数开高低收价，收益率，超额收益等
    
    :param tradeDate: 交易日起，格式为yyyymmdd,tradeDate、beginDate、endDate至少选择一个
    :param beginDate: 交易日起，格式为yyyymmdd,tradeDate、beginDate、endDate至少选择一个
    :param endDate: 交易日起，格式为yyyymmdd,tradeDate、beginDate、endDate至少选择一个
    :param secID: 通联编制的证券编码，可传入证券交易代码使用DataAPI.SecIDGet接口获取到。,可以是列表,可空
    :param ticker: 交易代码,可以是列表,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/idxMarket/getMktIdxdZH.csv?ispandas=1&') 
    try:
        tradeDate = tradeDate.strftime('%Y%m%d')
    except:
        tradeDate = tradeDate.replace('-', '')
    requestString.append("tradeDate=%s"%(tradeDate))
    try:
        beginDate = beginDate.strftime('%Y%m%d')
    except:
        beginDate = beginDate.replace('-', '')
    requestString.append("&beginDate=%s"%(beginDate))
    try:
        endDate = endDate.strftime('%Y%m%d')
    except:
        endDate = endDate.replace('-', '')
    requestString.append("&endDate=%s"%(endDate))
    requestString.append("&secID=")
    if hasattr(secID,'__iter__') and not isinstance(secID, str):
        if len(secID) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = secID
            requestString.append(None)
        else:
            requestString.append(','.join(secID))
    else:
        requestString.append(secID)
    requestString.append("&ticker=")
    if hasattr(ticker,'__iter__') and not isinstance(ticker, str):
        if len(ticker) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = ticker
            requestString.append(None)
        else:
            requestString.append(','.join(ticker))
    else:
        requestString.append(ticker)
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=False)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, 'MktIdxdZHGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=False)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, 'MktIdxdZHGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'secID', u'ticker', u'tradeDate', u'secShortNameEn', u'secShortNameEn', u'currencyCD', u'indexLevel', u'comGroup', u'comGroupEn', u'comGroupCD', u'comSubGroup', u'comSubGroupEN', u'comSubGroupCD', u'singleCom', u'singleComEn', u'singleComCD', u'LS', u'LSen', u'leverage', u'weightMetho', u'weightMethoEn', u'indexValueER', u'dailyIndexER', u'indexValueTR', u'dailyIndexTR', u'openIndex', u'highIndex', u'lowIndex', u'closeIndex', u'wtdIndexER', u'wtdIndexTR', u'mtdIndexER', u'mtdIndexTR', u'ytdIndexER', u'ytdIndexTR', u'indexOpenInt', u'turnoverVol', u'indexValueERUSD', u'dailyIndexERUSD', u'dailyIndexTRUSD', u'indexValueEREUR', u'dailyIndexEREUR', u'dailyIndexTREUR', u'indexValueERJPY', u'dailyIndexERJPY', u'dailyIndexTRJPY', u'indexValueERHKD', u'dailyIndexERHKD', u'dailyIndexTRHKD']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'secID': 'str','ticker': 'str','secShortNameEn': 'str','secShortNameEn': 'str','currencyCD': 'str','indexLevel': 'str','comGroup': 'str','comGroupEn': 'str','comGroupCD': 'str','comSubGroup': 'str','comSubGroupEN': 'str','comSubGroupCD': 'str','singleCom': 'str','singleComEn': 'str','singleComCD': 'str','LS': 'str','LSen': 'str','leverage': 'str','weightMetho': 'str','weightMethoEn': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def MktIdxdZHAnalyGet(tradeDate = "", beginDate = "", endDate = "", secID = "", ticker = "", field = "", pandas = "1"):
    """
    获取中汉指数日度分析数据，包含年化收益率，最大回撤，收益率标准差，夏普比率，索提诺比率等
    
    :param tradeDate: 交易日起，格式为yyyymmdd,tradeDate、beginDate、endDate至少选择一个
    :param beginDate: 交易日起，格式为yyyymmdd,tradeDate、beginDate、endDate至少选择一个
    :param endDate: 交易日起，格式为yyyymmdd,tradeDate、beginDate、endDate至少选择一个
    :param secID: 通联编制的证券编码，可传入证券交易代码使用DataAPI.SecIDGet接口获取到。,可以是列表,可空
    :param ticker: 交易代码,可以是列表,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/idxMarket/getMktIdxdZHAnaly.csv?ispandas=1&') 
    try:
        tradeDate = tradeDate.strftime('%Y%m%d')
    except:
        tradeDate = tradeDate.replace('-', '')
    requestString.append("tradeDate=%s"%(tradeDate))
    try:
        beginDate = beginDate.strftime('%Y%m%d')
    except:
        beginDate = beginDate.replace('-', '')
    requestString.append("&beginDate=%s"%(beginDate))
    try:
        endDate = endDate.strftime('%Y%m%d')
    except:
        endDate = endDate.replace('-', '')
    requestString.append("&endDate=%s"%(endDate))
    requestString.append("&secID=")
    if hasattr(secID,'__iter__') and not isinstance(secID, str):
        if len(secID) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = secID
            requestString.append(None)
        else:
            requestString.append(','.join(secID))
    else:
        requestString.append(secID)
    requestString.append("&ticker=")
    if hasattr(ticker,'__iter__') and not isinstance(ticker, str):
        if len(ticker) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = ticker
            requestString.append(None)
        else:
            requestString.append(','.join(ticker))
    else:
        requestString.append(ticker)
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=False)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, 'MktIdxdZHAnalyGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=False)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, 'MktIdxdZHAnalyGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'secID', u'ticker', u'tradeDate', u'secShortNameEn', u'secShortNameEn', u'currencyCD', u'indexLevel', u'comGroup', u'comGroupEn', u'comGroupCD', u'comSubGroup', u'comSubGroupEN', u'comSubGroupCD', u'singleCom', u'singleComEn', u'singleComCD', u'LS', u'LSen', u'leverage', u'weightMetho', u'weightMethoEn', u'indexER20d', u'indexTR20d', u'annuER20d', u'meanER20d', u'sd20d', u'maxDrawdown20d', u'annuShareRatioER20d', u'annuShareRatioTR20d', u'annuSortinoRatioER20d', u'annuSortinoRatioER20d', u'skewness20d', u'excessKurtosis20d']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'secID': 'str','ticker': 'str','secShortNameEn': 'str','secShortNameEn': 'str','currencyCD': 'str','indexLevel': 'str','comGroup': 'str','comGroupEn': 'str','comGroupCD': 'str','comSubGroup': 'str','comSubGroupEN': 'str','comSubGroupCD': 'str','singleCom': 'str','singleComEn': 'str','singleComCD': 'str','LS': 'str','LSen': 'str','leverage': 'str','weightMetho': 'str','weightMethoEn': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()




#!coding=utf-8
import datetime

def get_trade_date_list(begin_date, end_date):
    from . import DataAPI
    try:
        df = DataAPI.TradeCalGet(exchangeCD='XSHG', beginDate=begin_date, endDate=end_date, field='calendarDate,isOpen')
        date_list = list(df[df['isOpen'] == 1]['calendarDate'])
        return date_list
    except:
        print ('获取交易日历异常，请检查TradeCalGet的访问权限或者重试')
        return []

def check_sec_id(sec_id):
    if len(sec_id) == 11:
        if not sec_id.endswith('XSHG') and not sec_id.endswith('XSHE'):
            return False
        for i in range(6):
            if not sec_id[i].isdigit() and sec_id[i] != 'T':
                return False
        return True
    elif len(sec_id) == 9:
        if not sec_id.endswith('SZ') and not sec_id.endswith('SH'):
            return False
        if sec_id[0] == 'T':
            pass
        for i in range(6):
            if not sec_id[i].isdigit() and sec_id[i] != 'T':
                return False
        return True
    elif len(sec_id) == 6:
        for i in range(6):
            if not sec_id[i].isdigit() and sec_id[i] != 'T':
                return False
        return True
    return False

def load_df_from_file(file_path):
    try:
        import os
        if not os.path.exists(file_path):
            return None, 'csv文件不存在'
        import pandas as pd
        return pd.read_csv(file_path), ''
    except:
        return None, '错误的csv文件'

def load_df_from_data(data):
    try:
        import StringIO
        import pandas as pd
        return pd.read_csv(StringIO.StringIO(data)), ''
    except:
        return None, '错误的csv文件'

def check_trade_date_format(format, date_string):
    try:
        return datetime.datetime.strptime(str(date_string), format)
    except:
        return None

def convert_one_trade_date(date):
    dt = check_trade_date_format('%Y-%m-%d', date)
    if dt is not None:
        return dt
    dt = check_trade_date_format('%Y%m%d', date)
    if dt is not None:
        return dt
    dt = check_trade_date_format('%Y/%m/%d', date)
    if dt is not None:
        return dt
    return dt

def convert_trade_list(trade_date_list):
    result = []
    for date in trade_date_list:
        dt = convert_one_trade_date(date)
        if dt is not None:
            result.append(dt.strftime('%Y-%m-%d'))
        else:
            print ('交易日不合理：%s' % date)
            return []
    return result

def check_factor_data(df):
    trade_date_column_name = df.columns[0]
    trade_date_list = list(df[trade_date_column_name])
    trade_date_list = convert_trade_list(trade_date_list)
    if not trade_date_list:
        return False, '交易日检查未通过'
    # begin_date = trade_date_list[0]
    # end_date = trade_date_list[-1]
    # standard_date_list = get_trade_date_list(begin_date, end_date)
    # for idx, date in enumerate(trade_date_list):
    #     if date != standard_date_list[idx]:
    #         return False, '交易日不连续：%s' % (date)
    columns = list(df.columns)
    universe_len = len(columns) - 1
    for col in columns:
        if col != trade_date_column_name:
            if not check_sec_id(col):
                return False, '证券代码错误：%s' % col
    # nan_count = df.drop(trade_date_column_name, 1).isnull().sum(axis=1)
    # total = len(trade_date_list) * universe_len
    # none_count = 0
    # for idx, count in nan_count.iteritems():
    #     none_count += count
    #     if count > universe_len-300:
    #         return False, '交易日%s的因子覆盖率过低，覆盖数不到300' % (trade_date_list[idx])
    #
    # none_rate = float(float(none_count) / float(total))
    # if none_rate > 0.7:
    #     return False, '因子覆盖率过低：%0.2f%%，要求不低于30%%' % ((1-none_rate)*100)
    return True, ''